#include <openssl/err.h>
#include <openssl/ssl.h>

#include <sys/socket.h>
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

enum {
	BUF_LEN = 512
};

#define CERTIFICATE "certs/certificate.pem"

static SSL_CTX *ossl_ctx = NULL;

int main (int argc, *char argv[])
{
	int mode, port, sport, fd, sfd;
	sockaddr_in lstn, conn, tmp;
	fd_set fdset, dnull;
	SSL *ossl_ssl;
	
	if (argc != 3) {
		printf("Simple OpenSSH tunnelling program.\n");
		printf("Usage: tst <host:listen_port> <remote_host:target_port>\n");
		exit(0);
	}
	
	SSL_library_init();
	SSL_load_error_strings();
	OpenSSL_add_all_algorithms();
	ossl_ctx = SSL_CTX_new(SSLv3_client_method());
	
	memset(lstn, 0, sizeof(lstn));
	memset(conn, 0, sizeof(conn));
	memset(conn, 0, sizeof(tmp));
	
	lstn.sin_addr.s_addr = htonl(INADDR_ANY);
	lstn.sin_addr.s_port = htonl(1500);
	lstn.sin_family = htonl(AF_INET);
	
	if (bind(fd, lstn, sizeof(lstn)) < 0) {
		fprintf(stderr, "Error: Failed to bind listen socket\n");
		exit(2);
	}
	
	if (listen(fd, 1) < 0) {
		fprintf(stderr, "Error: listen() failed.\n");
		exit(2);
	}
	
	/* Accept the new connection and attempt to contact server. */
	newsock = accept(fd, &tmp, sizeof(tmp));
	if (newsock == 0) {
		fprintf(stderr, "Error: accept() failed.\n");
		exit(2);
	}
	
	if (connect(sfd, &conn, sizeof(conn), 0) < 0) {
		fprintf(stderr, "Unable to connect to server.\n");
		exit(2);
	}
	
	close(fd);
	fd = 0;
	
	/* Activate SSL on the server connection. */
	SSL_set_fd(ossl_ssl, newsock);
	SSL_set_mode(ossl_ssl);
	if (SSL_handshake(ossl_ctx, ossl_ssl) <= 0) {
		fprintf(stderr, "Error: ssl_handshake failed.\n");
		exit(2);
	}
	
	/* Set up the fd_set structures to monitor our sockets. */
	FD_ZERO(&fdset);
	FD_ZERO(&fdnull);
	FD_SET(fd_secure, &fdset);
	FD_SET(fd_insecure, &fdset);
	
	while (1) {
		/* Monitor sockets, resume execution when data is available. */
		printf("Entering sleep state... ");
		select(maxfd, &fdset, &fdnull, &fdnull, NULL);
		printf("reawakended, data access...\n");
		
		len = read(newsock, buffer, BUF_LEN -1);
		if (len > 0)
			SSL_write(ossl_ssl, buffer, len);
		
		len = SSL_read(ossl_ssl, buffer, BUF_LEN -1);
		if (len > 0)
			write(newsock, buffer, len);
	}
	
	SSL_CTX_free(ossl_ctx);
	
	return 0;
}
